% Example_02.m
% This program solves the problem posed in Example 2.
% It calculates the real gas specific volume from the
% SRK equation of state using the Newton-Raphson method
% for calculating the roots of a polynomial.
clear
clc
clf
% Input data
P = input(' Input the vector of pressure range (Pa) = ');
T = input(' Input temperature (K) = ');
R = 8314; % Gas constant (J/kmol.K)
Tc = input(' Critical temperature (K) = ');
Pc = input(' Critical pressure (Pa) = ');
omega = input(' Acentric factor = ');
% Constants of Soave-Redlich-Kwong equation of state
a = 0.4278 * R^2 * Tc^2 / Pc;
b = 0.0867 * R * Tc / Pc;
sc = [-0.15613, 1.55171, 0.48508];
s = polyval(sc,omega);
alpha = (1 + s * (1 - sqrt(T/Tc)))^2;
A = a * alpha * P / (R^2 * T^2);
B = b * P / (R * T);
for k = 1:length(P)
    % Defining the polynomial coefficients
    coef = [1, -1, A(k)-B(k)-B(k)^2, -A(k)*B(k)];
    v0(k) = R * T / P(k); % Ideal gas specific volume
    vol(k) = NRpoly(coef , 1, [], 2) * R * T / P(k); % Finding the root
    pause
end
% Show numerical results
fprintf('\nRESULTS:\n');
fprintf('Pres. = %5.2f Ideal gas vol. =%7.4f',P(1),v0(1));
fprintf(' Real gas vol. =%7.4f\n',vol(1));
for k=10:10:length(P)
fprintf('Pres. = %5.2f Ideal gas vol. =%7.4f',P(k),v0(k));
fprintf(' Real gas vol. =%7.4f\n',vol(k));
end
% plotting the results
loglog(P/1000,v0,'.',P/1000,vol,'LineWidth',2)
    set(gca,'LineWidth',2,'Fontsize',14)
xlabel('\bf Pressure, kPa','Fontsize',14)
ylabel('\bf Specific Volume, m^3/kmol','Fontsize',14)
title('\bf Newton-Raphson: fcn and path to root (*: initial; o: root)','Fontsize',14)
legend('\bf Ideal','\bf SRK','Fontsize',14)